
local MakePlayerCharacter = require "prefabs/player_common"


local assets = {

        Asset( "ANIM", "anim/player_basic.zip" ),
        Asset( "ANIM", "anim/player_idles_shiver.zip" ),
        Asset( "ANIM", "anim/player_actions.zip" ),
        Asset( "ANIM", "anim/player_actions_axe.zip" ),
        Asset( "ANIM", "anim/player_actions_pickaxe.zip" ),
        Asset( "ANIM", "anim/player_actions_shovel.zip" ),
        Asset( "ANIM", "anim/player_actions_blowdart.zip" ),
        Asset( "ANIM", "anim/player_actions_eat.zip" ),
        Asset( "ANIM", "anim/player_actions_item.zip" ),
        Asset( "ANIM", "anim/player_actions_uniqueitem.zip" ),
        Asset( "ANIM", "anim/player_actions_bugnet.zip" ),
        Asset( "ANIM", "anim/player_actions_fishing.zip" ),
        Asset( "ANIM", "anim/player_actions_boomerang.zip" ),
        Asset( "ANIM", "anim/player_bush_hat.zip" ),
        Asset( "ANIM", "anim/player_attacks.zip" ),
        Asset( "ANIM", "anim/player_idles.zip" ),
        Asset( "ANIM", "anim/player_rebirth.zip" ),
        Asset( "ANIM", "anim/player_jump.zip" ),
        Asset( "ANIM", "anim/player_amulet_resurrect.zip" ),
        Asset( "ANIM", "anim/player_teleport.zip" ),
        Asset( "ANIM", "anim/wilson_fx.zip" ),
        Asset( "ANIM", "anim/player_one_man_band.zip" ),
        Asset( "ANIM", "anim/shadow_hands.zip" ),
        Asset( "SOUND", "sound/sfx.fsb" ),
        Asset( "SOUND", "sound/wilson.fsb" ),
        Asset( "ANIM", "anim/beard.zip" ),
		
		Asset( "ANIM", "anim/my_own.zip" ),
}
local prefabs = {

}

-- Custom starting items
local start_inv = {

}

--Willow

local function sanityfn(inst)
	local x,y,z = inst.Transform:GetWorldPosition()	
	local delta = 0
	local max_rad = 10
	local ents = TheSim:FindEntities(x,y,z, max_rad, {"fire"})
    for k,v in pairs(ents) do 
    	if v.components.burnable and v.components.burnable.burning then
    		local sz = TUNING.SANITYAURA_TINY
    		local rad = v.components.burnable:GetLargestLightRadius() or 1
    		sz = sz * ( math.min(max_rad, rad) / max_rad )
			local distsq = inst:GetDistanceSqToInst(v)
			delta = delta + sz/math.max(1, distsq)
    	end
    end

    if GetWorld().IsVolcano() then
    	local map = GetWorld().Map
    	if map and map:IsTileGridValid() then
			local tx, ty = map:GetTileXYAtPoint(x, y, z)
			local dist = map:GetClosestTileDist(tx, ty, GROUND.VOLCANO_LAVA, 4)
			if dist <= 4 then
				delta = math.max(delta, TUNING.SANITYAURA_TINY * (1 - (dist / 4)))
			end
		end
    end
    
    return delta
end


local function boatFireUpdate(inst, dt)
	--print(dt)
	if not inst.components.driver:GetIsDriving() then 
		--put out the fire 
		inst.components.burnable:Extinguish()
	else 
		local boatHurtRate = -10
		local amount = boatHurtRate * dt
		inst.components.driver.vehicle.components.boathealth:DoDelta(amount, "fire")
	end 



end 


local function onIgnite(inst)
	if inst.burnTask then 
		inst.burnTask:Cancel()
	end
	inst.burnTask = inst:DoPeriodicTask(0.5, function() boatFireUpdate(inst, .5) end)
end 

local function onExtinguish(inst)
	if inst.burnTask then 
		inst.burnTask:Cancel()
	end 
end
-- Wigfrid
local smallScale = 0.5
local medScale = 0.7
local largeScale = 1.1

local function onkill(inst, data)
	if data.cause == inst.prefab 
		and not data.inst:HasTag("prey") 
		and not data.inst:HasTag("veggie") 
		and not data.inst:HasTag("structure") then
		local delta = (data.inst.components.combat.defaultdamage) * 0.25
        inst.components.health:DoDelta(delta, false, "battleborn")
        inst.components.sanity:DoDelta(delta)

        if math.random() < .1 and not data.inst.components.health.nofadeout then
        	local time = data.inst.components.health.destroytime or 2
        	inst:DoTaskInTime(time, function()
        		local s = medScale
        		if data.inst:HasTag("smallcreature") then
        			s = smallScale
    			elseif data.inst:HasTag("largecreature") then
    				s = largeScale
    			end
        		local fx = SpawnPrefab("wathgrithr_spirit")
        		fx.Transform:SetPosition(data.inst:GetPosition():Get())
        		fx.Transform:SetScale(s,s,s)
    		end)
        end

	end
end

local function dorainsparks(inst, dt)

    if (inst.components.moisture and inst.components.moisture:GetMoisture() > 0) then
    	inst.spark_time = inst.spark_time - dt

    	if inst.spark_time <= 0 then
    		
    		--GetClock():DoLightningLighting()
    		inst.spark_time = 3+math.random()*2

    		local pos = Vector3(inst.Transform:GetWorldPosition())
    		local damage = nil

    		-- Raining, no moisture-giving equipment on head, and moisture is increasing. Pro-rate damage based on waterproofness.
    		if GetSeasonManager():IsRaining() and inst.components.inventory:GetEquippedMoistureRate(EQUIPSLOTS.HEAD) <= 0 and inst.components.moisture:GetDelta() > 0 then
	    		local waterproofmult = (inst.components.moisture and inst.components.moisture.sheltered and inst.components.inventory) and (1 - (inst.components.inventory:GetWaterproofness() + inst.components.moisture.shelter_waterproofness)) or (inst.components.inventory and (1 - inst.components.inventory:GetWaterproofness()) or 1)
	    		damage = waterproofmult > 0 and math.min(TUNING.WX78_MIN_MOISTURE_DAMAGE, TUNING.WX78_MAX_MOISTURE_DAMAGE * waterproofmult) or 0
	    		inst.components.health:DoDelta(damage, false, "rain")
				pos.y = pos.y + 1 + math.random()*1.5
	    	else -- We have moisture-giving equipment on our head or it is not raining and we are just passively wet (but drying off). Do full damage.
	    		if inst.components.moisture:GetDelta() >= 0 then -- Moisture increasing (wearing something moisturizing)
	    			inst.components.health:DoDelta(TUNING.WX78_MAX_MOISTURE_DAMAGE, false, "water")
	    		else -- Drying damage
	    			inst.components.health:DoDelta(TUNING.WX78_MOISTURE_DRYING_DAMAGE, false, "water")
	    		end
				pos.y = pos.y + .25 + math.random()*2
	    	end
			
			if not damage or (damage and damage < 0) then
				local spark = SpawnPrefab("sparks")
				spark.Transform:SetPosition(pos:Get())
			end
    	end
    end

end

local fn = function(inst)

    if TUNING.OWN_PERK1 == "dont_burn" or TUNING.OWN_PERK2 == "dont_burn" then
        inst.components.health.fire_damage_scale = 0
    end
	
    if TUNING.OWN_PERK1 == "sanity_fire" or TUNING.OWN_PERK2 == "sanity_fire" then
        inst.components.sanity.custom_rate_fn = sanityfn
	    inst:AddTag("flingomatic_freeze_immune")
	    inst:AddComponent("burnable")
        inst.components.burnable:SetFXLevel(2)
        inst.components.burnable:SetBurnTime(5)
        inst.components.burnable:AddBurnFX("character_fire",Vector3(0, 0, 0) )
        inst.components.burnable.canlight = false 
        inst.components.burnable:MakeNotWildfireStarter()
        inst.components.burnable:SetOnIgniteFn(onIgnite)
        inst.components.burnable:SetOnExtinguishFn(onExtinguish)
        inst.components.burnable.lightningimmune = true
    end
	
	if TUNING.OWN_PERK1 == "can_abigail" or TUNING.OWN_PERK2 == "can_abigail" then
	    inst:DoTaskInTime(0, function() 
    		local found = false
    		for k,v in pairs(Ents) do
    			if v.prefab == "abigail" then
    				found = true
    				break
    			end
    		end
    		if not found then
    			inst.components.inventory:GuaranteeItems(prefabs)
    		end
    	end)
	end
	
	if TUNING.OWN_PERK1 == "can_books" or TUNING.OWN_PERK2 == "can_books" then
	
	    inst:AddComponent("reader")
		
	    local booktab = {str = STRINGS.TABS.BOOKS, sort=999, icon = "tab_book.tex"}
	    inst.components.builder:AddRecipeTab(booktab)
	    Recipe("book_birds", {Ingredient("papyrus", 2), Ingredient("bird_egg", 2)}, booktab, {SCIENCE = 0, MAGIC = 0, ANCIENT = 0})
	    Recipe("book_gardening", {Ingredient("papyrus", 2), Ingredient("seeds", 1), Ingredient("poop", 1)}, booktab, {SCIENCE = 1})
	    Recipe("book_sleep", {Ingredient("papyrus", 2), Ingredient("nightmarefuel", 2)}, booktab, {MAGIC = 2})
	    Recipe("book_brimstone", {Ingredient("papyrus", 2), Ingredient("redgem", 1)}, booktab, {MAGIC = 3})
	    Recipe("book_tentacles", {Ingredient("papyrus", 2), Ingredient("tentaclespots", 1)}, booktab, {SCIENCE = 3})
	end
	
	if TUNING.OWN_PERK1 == "monster" or TUNING.OWN_PERK2 == "monster" then
		inst:AddTag("spiderwhisperer")
	    inst:AddTag("monster")
	end

	if TUNING.OWN_PERK1 == "heal_kill" or TUNING.OWN_PERK2 == "heal_kill" then
	    inst:ListenForEvent("continuefrompause", function()
		    if Profile:IsWathgrithrFontEnabled() then
			    inst.components.talker.font = TALKINGFONT_WATHGRITHR
		    else
			    inst.components.talker.font = TALKINGFONT
		    end
	    end, GetWorld())
		inst:ListenForEvent("entity_death", function(wrld, data) onkill(inst, data) end, GetWorld())
	end
	
	if TUNING.OWN_PERK1 == "cant_sleep" or TUNING.OWN_PERK2 == "cant_sleep" then
	    inst:AddTag("insomniac")
	end
	
	if TUNING.OWN_PERK1 == "eat_meat" or TUNING.OWN_PERK2 == "eat_meat" then
	    inst.components.eater:SetCarnivore(true)
	end
	
	if TUNING.OWN_PERK1 == "monkey_walk" or TUNING.OWN_PERK2 == "monkey_walk" then
	    inst:AddTag("wilbur")
	    inst:AddTag("monkey")
		inst.components.locomotor:AddSpeedModifier_Additive("WILBUR_WALK", TUNING.WILBUR_WALK_SPEED_PENALTY)
	end
	
	if TUNING.OWN_PERK1 == "sciene_plas" or TUNING.OWN_PERK2 == "sciene_plas" then
        inst.components.builder.science_bonus = 1
	end
	
	if TUNING.OWN_PERK1 == "hates_spoilage" or TUNING.OWN_PERK2 == "hates_spoilage" then
        inst.components.eater.stale_hunger = TUNING.WICKERBOTTOM_STALE_FOOD_HUNGER
        inst.components.eater.stale_health = TUNING.WICKERBOTTOM_STALE_FOOD_HEALTH
        inst.components.eater.spoiled_hunger = TUNING.WICKERBOTTOM_SPOILED_FOOD_HUNGER
        inst.components.eater.spoiled_health = TUNING.WICKERBOTTOM_SPOILED_FOOD_HEALTH
	end
	
	if TUNING.OWN_PERK1 == "water_dam" or TUNING.OWN_PERK2 == "water_dam" then
	    inst.spark_time = 3
        inst:DoPeriodicTask(1/10, function() dorainsparks(inst, 1/10) end)
	end
	
    if TUNING.OWN_ITEM1 ~= "nope" then
        table.insert(start_inv, TUNING.OWN_ITEM1)
    end

    if TUNING.OWN_ITEM2 ~= "nope" then
        table.insert(start_inv, TUNING.OWN_ITEM2)
    end

    if TUNING.OWN_ITEM3 ~= "nope" then
        table.insert(start_inv, TUNING.OWN_ITEM3)
    end

	inst.soundsname = TUNING.OWN_TALK_SOUND
	
	if TUNING.OWN_TALK_SOUND == "webber" or TUNING.OWN_TALK_SOUND == "wathgrithr" then
	    inst.talker_path_override = "dontstarve_DLC001/characters/"
	end
	
	if TUNING.OWN_TALK_SOUND == "walani" or TUNING.OWN_TALK_SOUND == "warly" or TUNING.OWN_TALK_SOUND == "wilbur" or TUNING.OWN_TALK_SOUND == "woodlegs" then
	    inst.talker_path_override = "dontstarve_DLC002/characters/"
	end

	inst.MiniMapEntity:SetIcon( TUNING.OWN_MAP..".png" )
		
	inst.components.health:SetMaxHealth(TUNING.OWN_HP)
	inst.components.hunger:SetMax(TUNING.OWN_HUNGER)
	inst.components.sanity:SetMax(TUNING.OWN_SANITY)
	
    inst.components.combat.damagemultiplier = 1
	
	inst.components.hunger.hungerrate = 1 * TUNING.WILSON_HUNGER_RATE
	
	inst.components.locomotor.walkspeed = 4
	inst.components.locomotor.runspeed = 6
	
	inst.AnimState:OverrideSymbol("headbase", TUNING.OWN_HEAD, "headbase")
	inst.AnimState:OverrideSymbol("headbase_hat", TUNING.OWN_HEAD, "headbase_hat")
	inst.AnimState:OverrideSymbol("hair", TUNING.OWN_HAIR2, "hair")
	inst.AnimState:OverrideSymbol("hair_hat", TUNING.OWN_HAIR2, "hair_hat")
	inst.AnimState:OverrideSymbol("hairpigtails", TUNING.OWN_HAIR, "hairpigtails")
	inst.AnimState:OverrideSymbol("face", TUNING.OWN_FACE, "face")
	inst.AnimState:OverrideSymbol("cheeks", TUNING.OWN_CHEEKS, "cheeks")
	inst.AnimState:OverrideSymbol("torso", TUNING.OWN_TORSO, "torso")
	inst.AnimState:OverrideSymbol("torso_pelvis", TUNING.OWN_TORSO_PELVIS, "torso_pelvis")
	inst.AnimState:OverrideSymbol("arm_upper", TUNING.OWN_ARM_UP, "arm_upper")
	inst.AnimState:OverrideSymbol("arm_lower", TUNING.OWN_ARM_LOW, "arm_lower")
	inst.AnimState:OverrideSymbol("hand", TUNING.OWN_HANDS, "hand")
	inst.AnimState:OverrideSymbol("skirt", TUNING.OWN_SKIRT, "skirt")
	inst.AnimState:OverrideSymbol("leg", TUNING.OWN_LEGS, "leg")
	inst.AnimState:OverrideSymbol("foot", TUNING.OWN_FOOTS, "foot")
	
	
end

return MakePlayerCharacter("my_own", prefabs, assets, fn, start_inv)
